"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../lib/util");
const queryUtils_1 = require("../lib/queryUtils");
const errors_1 = require("../lib/errors");
const queryItemConfigs_1 = require("../lib/queryItemConfigs");
const queriesController = (db, service, urlPrefix) => {
    service.app.get(`${urlPrefix}/queries`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const { project, universe, dashboard } = req.query;
        if (!project || !universe || !dashboard) {
            return res.json(errors_1.getError(4, 'One or more missing query params: project, universe, dashboard'));
        }
        const dash = await db('dashboards').where({ name: dashboard }).first('*');
        const gridLayout = JSON.parse(dash.grid);
        const queries = Object.values(await queryItemConfigs_1.getDashboardQueryItemsWithConfigs(db, dash.id, universe, project));
        for (const q of queries) {
            q.layouts = JSON.parse(q.layouts);
        }
        const layouts = queries.reduce((acc, q) => {
            for (const breakpoint in q.layouts) {
                acc[breakpoint] = acc[breakpoint] || [];
                acc[breakpoint].push(q.layouts[breakpoint]);
            }
            return acc;
        }, {});
        return res.status(200).json({
            version: process.env.npm_package_version,
            project_name: project,
            universe_name: universe,
            queries: queries,
            layouts,
            gridLayout,
        });
    }));
    service.app.post(`${urlPrefix}/queries/:id`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const token = req.headers['x-coroner-token'];
        if (!token) {
            return res.status(401).send(errors_1.getError(2, 'Please provide a coroner token'));
        }
        const { project, universe } = req.query;
        if (!project || !universe) {
            return res.json(errors_1.getError(4, 'One or more missing query params: project, universe'));
        }
        const queryOverrides = req.body;
        const coronerUrl = util_1.getCoronerUrl(universe, service, req);
        const id = parseInt(req.params.id, 10);
        if (!id) {
            return res.json(errors_1.getError(3));
        }
        const queryItem = await db('query_items').first('*').where({ id });
        const queryConfigs = await queryItemConfigs_1.getProjectQueryConfigs(db, id, universe, project);
        const apiConfigData = await util_1.getApiConfig(req, token, service, universe);
        const [queryForRequest, crdbResp] = await queryUtils_1.execQuery(queryItem, queryOverrides, queryConfigs, coronerUrl, token, project, universe, apiConfigData);
        return res.status(200).json(Object.assign({ query: queryForRequest }, crdbResp));
    }));
    service.app.post(`${urlPrefix}/queries/:id/config`, service.identityManager.authenticateRequest(), util_1.accessCheck(service), util_1.adminCheck(service), util_1.asyncWrapper(async (req, res, _next) => {
        const token = req.headers['x-coroner-token'];
        if (!token) {
            return res.status(401).send(errors_1.getError(2, 'Please provide a coroner token'));
        }
        const { key, value } = req.body;
        if (!key || !value) {
            return res.json(errors_1.getError(4, 'One or more missing body params: key, value'));
        }
        const id = parseInt(req.params.id, 10);
        if (!Number.isInteger(id)) {
            return res.status(422).send(errors_1.getError(3, 'Invalid ID'));
        }
        const queryItem = await db('query_items').first('*').where({ id });
        if (!queryItem) {
            return res.status(422).send(errors_1.getError(3, 'Invalid ID'));
        }
        const { project, universe } = req.query;
        if (!project || !universe) {
            return res.json(errors_1.getError(4, 'One or more missing query params: project, universe'));
        }
        await queryItemConfigs_1.createOrUpdateQueryConfig(db, queryItem.id, key, value, {
            universe_name: universe,
            project_name: project,
        });
        return res.status(200).json({});
    }));
};
exports.default = queriesController;
//# sourceMappingURL=queries.js.map